#!/bin/sh

# some usefull things (thanks to oz_paulb from mazda3revolution.com - your code is awesome! I wish, I could understand everything ...)

get_cmu_sw_version()
{
	_ver=$(/bin/grep "^JCI_SW_VER=" /jci/version.ini | /bin/sed 's/^.*_\([^_]*\)\"$/\1/')
	_patch=$(/bin/grep "^JCI_SW_VER_PATCH=" /jci/version.ini | /bin/sed 's/^.*\"\([^\"]*\)\"$/\1/')
	_flavor=$(/bin/grep "^JCI_SW_FLAVOR=" /jci/version.ini | /bin/sed 's/^.*_\([^_]*\)\"$/\1/')

	if [[ ! -z "${_flavor}" ]]; then
		echo "${_ver}${_patch}-${_flavor}"
	else
		echo "${_ver}${_patch}"
	fi
}

get_cmu_sw_version_only()
{
	_veronly=$(/bin/grep "^JCI_SW_VER=" /jci/version.ini | /bin/sed 's/^.*_\([^_]*\)\"$/\1/')
	echo "${_veronly}"
}

log_message()
{
	echo "$*" 1>2
	echo -e "$*" >> "${MYDIR}/AIO_log.txt"
	/bin/fsync "${MYDIR}/AIO_log.txt"
}


show_message()
{
	sleep 4
	killall jci-dialog
	log_message "= POPUP: $* "
	/jci/tools/jci-dialog --info --title="MESSAGE" --text="$*" --no-cancel &
}


show_message_OK()
{
	sleep 4
	killall jci-dialog
	log_message "= POPUP: $* "
	/jci/tools/jci-dialog --confirm --title="CONTINUE INSTALLATION?" --text="$*" --ok-label="YES - GO ON" --cancel-label="NO - ABORT"
	if [ $? != 1 ]
		then
			killall jci-dialog
			break
		else
			show_message "INSTALLATION ABORTED! PLEASE UNPLUG USB DRIVE"
			sleep 5
			exit
		fi
}

add_app_json()
# script by vic_bam85
{	
	# check if entry in additionalApps.json still exists, if so nothing is to do
	count=$(grep -c '{ "name": "'"${1}"'"' /jci/opera/opera_dir/userjs/additionalApps.json)
	if [ "$count" = "0" ]
		then
			log_message "=== No entry of ${2} found in additionalApps.json, seems to be the first installation ==="
			mv /jci/opera/opera_dir/userjs/additionalApps.json /jci/opera/opera_dir/userjs/additionalApps.json.old
			# delete last line with "]" from additionalApps.json
			echo "$(cat /jci/opera/opera_dir/userjs/additionalApps.json.old)" | grep -v "]" > /jci/opera/opera_dir/userjs/additionalApps.json		
			# check, if other entrys exists
			count=$(grep -c '}' /jci/opera/opera_dir/userjs/additionalApps.json)
			if [ "$count" != "0" ]
				then
					# if so, add "," to the end of last line to additionalApps.json
					echo "$(cat /jci/opera/opera_dir/userjs/additionalApps.json)", > /jci/opera/opera_dir/userjs/additionalApps.json
					log_message "=== Found existing entrys in additionalApps.json ==="
			fi
			# add app entry and "]" again to last line of additionalApps.json
			log_message "=== Add ${2} to last line of additionalApps.json ==="
			echo '{ "name": "'"${1}"'", "label": "'"${2}"'" }' >> /jci/opera/opera_dir/userjs/additionalApps.json
			echo "]" >> /jci/opera/opera_dir/userjs/additionalApps.json
		else
			log_message "=== ${2} still exists in additionalApps.json, no modification necessary ==="
	fi
}

# disable watchdog and allow write access
echo 1 > /sys/class/gpio/Watchdog\ Disable/value
mount -o rw,remount /


MYDIR=$(dirname $(readlink -f $0))
CMU_SW_VER=$(get_cmu_sw_version)
CMU_VER_ONLY=$(get_cmu_sw_version_only)
rm -f "${MYDIR}/AIO_log.txt"


log_message "=== START LOGGING ... ==="
log_message "=== CMU_SW_VER = ${CMU_SW_VER} ==="
log_message "=== MYDIR = ${MYDIR} ==="
log_message "=== Watchdog temporary disabeld and write access enabled ==="


# first test, if copy from MZD to sd card is working to test correct mount point
cp /jci/sm/sm.conf ${MYDIR}
if [ -e ${MYDIR}/sm.conf ]
	then
		log_message "=== Copytest to sd card successful, mount point is OK ==="
		rm -f ${MYDIR}/sm.conf
	else
		log_message "=== Copytest to sd card not successful, mount point not found! ==="
		/jci/tools/jci-dialog --title="ERROR!" --text="Mount point not found, have to reboot again" --ok-label='OK' --no-cancel &
		sleep 5
		reboot
		exit
fi


show_message_OK "Version = ${CMU_SW_VER} : To continue installation press OK"


# a window will appear for 4 seconds to show the beginning of installation
show_message "START OF TWEAK INSTALLATION ..."


# disable watchdogs in /jci/sm/sm.conf to avoid boot loops if somthing goes wrong
if [ ! -e /jci/sm/sm.conf.org ]
	then
		cp -a /jci/sm/sm.conf /jci/sm/sm.conf.org
		log_message "=== Backup of /jci/sm/sm.conf to sm.conf.org ==="
	else log_message "=== Backup of /jci/sm.conf.org already there! ==="
fi
sed -i 's/watchdog_enable="true"/watchdog_enable="false"/g' /jci/sm/sm.conf
sed -i 's|args="-u /jci/gui/index.html"|args="-u /jci/gui/index.html --noWatchdogs"|g' /jci/sm/sm.conf
log_message "=== WATCHDOG IN SM.CONF PERMANENTLY DISABLED ==="


# -- Enable userjs and allow file XMLHttpRequest in /jci/opera/opera_home/opera.ini - backup first - then edit
if [ ! -e /jci/opera/opera_home/opera.ini.org ]
	then
		cp -a /jci/opera/opera_home/opera.ini /jci/opera/opera_home/opera.ini.org
		log_message "=== Backup of /jci/opera/opera_home/opera.ini to opera.ini.org ==="
	else log_message "=== Backup of /jci/opera/opera_home/opera.ini.org already there! ==="
fi
sed -i 's/User JavaScript=0/User JavaScript=1/g' /jci/opera/opera_home/opera.ini
count=$(grep -c "Allow File XMLHttpRequest=" /jci/opera/opera_home/opera.ini)
if [ "$count" = "0" ]
	then
		sed -i '/User JavaScript=.*/a Allow File XMLHttpRequest=1' /jci/opera/opera_home/opera.ini
	else
		sed -i 's/Allow File XMLHttpRequest=.*/Allow File XMLHttpRequest=1/g' /jci/opera/opera_home/opera.ini
fi
log_message "=== ENABLED USERJS AND ALLOWED FILE XMLHTTPREQUEST IN /JCI/OPERA/OPERA_HOME/OPERA.INI  ==="

# Remove fps.js if still exists
if [ -e /jci/opera/opera_dir/userjs/fps.js ]
	then mv /jci/opera/opera_dir/userjs/fps.js /jci/opera/opera_dir/userjs/fps.js.org
	log_message "=== Moved /jci/opera/opera_dir/userjs/fps.js to fps.js.org  ==="
fi

# Enable WIFI
if grep -Fq "(framework.localize.REGIONS['NorthAmerica'])" /jci/gui/apps/syssettings/js/syssettingsApp.js
	then
		echo "exist"
		log_message "=== Wifi already active for NA ==="
	else
		sed -i 's/(framework.localize.REGIONS['NorthAmerica'])/(framework.localize.REGIONS['Japan'])/g' /jci/gui/apps/syssettings/js/syssettingsApp.js
		log_message "=== Wifi for NA enabled ==="
		break
fi

# FuelConsumptionTweak
show_message "INSTALL FUEL CONSUMPTION TWEAK ..."
if [ ! -e /jci/gui/apps/ecoenergy/controls/FuelConsumption/css/FuelConsumptionCtrl.css.org ]
	then
		cp -a /jci/gui/apps/ecoenergy/controls/FuelConsumption/css/FuelConsumptionCtrl.css /jci/gui/apps/ecoenergy/controls/FuelConsumption/css/FuelConsumptionCtrl.css.org
		log_message "=== Backup of /jci/gui/apps/ecoenergy/controls/FuelConsumption/css/FuelConsumptionCtrl.css to FuelConsumptionCtrl.css.org ==="
fi
if [ ! -e /jci/gui/apps/ecoenergy/controls/FuelConsumption/images/FuelConsBG.png.org ]
	then
		cp -a /jci/gui/apps/ecoenergy/controls/FuelConsumption/images/FuelConsBG.png /jci/gui/apps/ecoenergy/controls/FuelConsumption/images/FuelConsBG.png.org
		log_message "=== Backup of /jci/gui/apps/ecoenergy/controls/FuelConsumption/images/FuelConsBG.png to FuelConsBG.png.org ==="
fi
if [ ! -e /jci/gui/apps/ecoenergy/controls/FuelConsumption/js/FuelConsumptionCtrl.js.org ]
	then
		cp -a /jci/gui/apps/ecoenergy/controls/FuelConsumption/js/FuelConsumptionCtrl.js /jci/gui/apps/ecoenergy/controls/FuelConsumption/js/FuelConsumptionCtrl.js.org
		log_message "=== Backup of /jci/gui/apps/ecoenergy/controls/FuelConsumption/js/FuelConsumptionCtrl.js to FuelConsumptionCtrl.js.org ==="
fi

cp -a ${MYDIR}/config/FuelConsumptionTweak/jci/gui/apps/ecoenergy/controls/FuelConsumption/css/FuelConsumptionCtrl.css /jci/gui/apps/ecoenergy/controls/FuelConsumption/css/
log_message "=== Copied /jci/gui/apps/ecoenergy/controls/FuelConsumption/css/FuelConsumptionCtrl.css ==="
cp -a ${MYDIR}/config/FuelConsumptionTweak/jci/gui/apps/ecoenergy/controls/FuelConsumption/images/FuelConsBG.png /jci/gui/apps/ecoenergy/controls/FuelConsumption/images/
log_message "=== Copied /jci/gui/apps/ecoenergy/controls/FuelConsumption/images/FuelConsBG.png ==="
cp -a ${MYDIR}/config/FuelConsumptionTweak/jci/gui/apps/ecoenergy/controls/FuelConsumption/js/FuelConsumptionCtrl.js /jci/gui/apps/ecoenergy/controls/FuelConsumption/js/
log_message "=== Copied /jci/gui/apps/ecoenergy/controls/FuelConsumption/js/FuelConsumptionCtrl.js ==="
log_message "=== END INSTALLATION OF FUEL CONSUMPTION TWEAK ==="

# Get_CID_Tweak
if [ -e /mnt/sd_nav/content ]
    then
		show_message "GET SD-CARD CID  ..."
        cp ${MYDIR}/config/get_sd_cid/cmu_sd_cid_getter_v3 /jci/nng/cmu_sd_cid_getter
		/bin/fsync /jci/nng/cmu_sd_cid_getter
		chmod 755 /jci/nng/cmu_sd_cid_getter
		log_message "=== Copied cmu_sd_cid_getter to /jci/nng/ ==="
		cd /jci/nng
		./cmu_sd_cid_getter /dev/sda1 >>SD_CID.txt
		cp /jci/nng/SD_CID.txt ${MYDIR}/SD_CID.txt
		cp /jci/nng/SD_CID.txt /mnt/sd_nav/SD_CID.txt
		rm -f /jci/nng/cmu_sd_cid_getter
		rm -f /jci/nng/SD_CID.txt
fi
log_message "=== END of Get SD-Card CID ==="

log_message "=== END OF TWEAKS INSTALLATION ==="

# a window will appear for asking to reboot automatically
sleep 2
killall jci-dialog
sleep 2
/jci/tools/jci-dialog --confirm --title="SELECTED ALL-IN-ONE TWEAKS APPLIED" --text="Click OK to reboot the system"
		if [ $? != 1 ]
		then
			sleep 8
			reboot
			exit
		fi
sleep 2
killall jci-dialog
